import java.awt.*;
import java.net.*;
import java.io.*;
import SmtpURLConnection;
import SmtpURLStreamHandler;
import SmtpURLStreamHandlerFactory;

public class EX17A extends Frame
{
    TextField from, to, smtpServer, status;
    TextArea note;
    Button send, close;

    public EX17A(String caption)
    {
        super(caption);

        from = new TextField(20);
        to = new TextField(20);
        smtpServer = new TextField(40);
        note = new TextArea(10, 40);
        status = new TextField(40);
        status.setEditable(false);
        send = new Button("Send");
        close = new Button("Close");
        Label l;

        GridBagLayout gbag = new GridBagLayout();
        GridBagConstraints cons = new GridBagConstraints();
        cons.insets = new Insets(5, 5, 5, 5);

        setLayout(gbag);

        cons.anchor = GridBagConstraints.NORTHWEST;
        l = new Label("From:");
        gbag.setConstraints(l, cons);
        add(l);

        cons.gridwidth = GridBagConstraints.REMAINDER;
        cons.fill = GridBagConstraints.HORIZONTAL;
        gbag.setConstraints(from, cons);
        add(from);

        cons.gridwidth = 1;
        cons.fill = GridBagConstraints.NONE;
        l = new Label("To:");
        gbag.setConstraints(l, cons);
        add(l);

        cons.gridwidth = GridBagConstraints.REMAINDER;
        cons.fill = GridBagConstraints.HORIZONTAL;
        gbag.setConstraints(to, cons);
        add(to);

        cons.gridwidth = 1;
        cons.fill = GridBagConstraints.NONE;
        l = new Label("SMTP Server:");
        gbag.setConstraints(l, cons);
        add(l);

        cons.gridwidth = GridBagConstraints.REMAINDER;
        cons.fill = GridBagConstraints.HORIZONTAL;
        gbag.setConstraints(smtpServer, cons);
        add(smtpServer);

        cons.fill = GridBagConstraints.NONE;
        cons.insets.bottom = 0;
        l = new Label("Note:");
        gbag.setConstraints(l, cons);
        add(l);
        cons.insets.bottom = 5;

        cons.weighty = 1.0;
        cons.fill = GridBagConstraints.BOTH;
        gbag.setConstraints(note, cons);
        add(note);

        cons.gridwidth = 1;
        cons.fill = GridBagConstraints.NONE;
        cons.weighty = 0;
        l = new Label("Status:");
        gbag.setConstraints(l, cons);
        add(l);

        cons.gridwidth = GridBagConstraints.REMAINDER;
        cons.fill = GridBagConstraints.HORIZONTAL;
        cons.weightx = 1.0;
        gbag.setConstraints(status, cons);
        add(status);

        Panel buttons = new Panel();
        buttons.setLayout(new FlowLayout(FlowLayout.CENTER, 20, 5));
        buttons.add(send);
        buttons.add(close);
        cons.gridwidth = GridBagConstraints.REMAINDER;
        cons.fill = GridBagConstraints.NONE;
        cons.anchor = GridBagConstraints.CENTER;
        gbag.setConstraints(buttons, cons);
        add(buttons);
        
        pack();
        show();

        URL.setURLStreamHandlerFactory(new SmtpURLStreamHandlerFactory());
    }
    
    public boolean handleEvent(Event evt)
    {
        if (evt.target == send)
        {
            if (to.getText().length() > 0 && smtpServer.getText().length() > 0)
                send();
            else
                status.setText("Enter destination address and SMTP Server.");
            return true;
        }
        else if (evt.target == close || evt.id == Event.WINDOW_DESTROY)
        {
            dispose();
            System.exit(0);
            return true;
        }
        else
            return super.handleEvent(evt);
    }

    protected void send()
    {
        SmtpURLConnection mailConn = null;
        status.setText("Sending note...");
        try
        {
            URL url = new URL("mailto:" + to.getText());
            mailConn = (SmtpURLConnection)url.openConnection();
            mailConn.setFrom(from.getText());
            mailConn.setSmtpServer(smtpServer.getText());
            
            PrintStream os = new PrintStream(mailConn.getOutputStream());
            os.print(note.getText());
            mailConn.sendIt();

            status.setText("Note successfully sent to " + to.getText());
        }
        catch (MalformedURLException e)
            status.setText("URL Error: " + e);
        catch (UnknownHostException e)
            status.setText("Host could not be found.  Try again.");
        catch (IOException e)
            status.setText("Stream Error: " + e);
        finally
        {
            if (mailConn != null)
            {
                try
                    mailConn.closeServer();
                catch (IOException e) {}
            }
        }
    }

    public static void main(String args[])
    {
        new EX17A("Java SMTP Client");
    }
}
